@@ -25,7 +25,7 @@
 #
 #  Created by Anton Berezin  <tobez@plab.ku.dk>
 #
-#  $Id: Application.pm,v 1.30 2007/08/03 19:23:48 dk Exp $
+#  $Id: Application.pm,v 1.31 2008/04/23 08:18:49 dk Exp $
 
 package main;
 
@@ -679,6 +679,11 @@ link POD format ( see L<perlpod> ) - the string is treated
 as "manpage/section", where 'manpage' is the file with POD 
 content and 'section' is the topic inside the manpage.
 
+=item sync
+
+Synchronizes all pending requests where there are any. Is
+an effective C<XSync(false)> on X11, and is a no-op otherwise.
+
 =item sys_action CALL
 
 CALL is an arbitrary string of the system
