@@ -27,7 +27,7 @@
 #     Dmitry Karasik <dk@plab.ku.dk> 
 #     Anton Berezin  <tobez@plab.ku.dk>
 #
-#  $Id: Edit.pm,v 1.50 2007/09/13 15:12:24 dk Exp $
+#  $Id: Edit.pm,v 1.52 2008/10/29 19:40:52 dk Exp $
 #
 # edit block types
 package bt;
@@ -549,10 +549,7 @@ sub on_paint
 		$canvas-> clipRect( $a[0], $a[1], $a[2] - 1, $a[3] - 1);
 		$canvas-> bar( 0, 0, @size);
 	} else {
-		$canvas-> rect3d( 
-			0, 0, $size[0]-1, $size[1]-1, $bw, 
-			$self-> dark3DColor, $self-> light3DColor, $clr[1]
-		);
+		$self-> draw_border( $canvas, $clr[1], @size);
 		# $canvas-> clipRect( $bw, $bw + $dy, $size[0] - $bw - $dx - 1, $size[1] - $bw - 1);
 		$canvas-> clipRect( $a[0], $a[1], $a[2] - 1, $a[3] - 1);
 	}
@@ -946,7 +943,7 @@ sub on_keydown
 		$mod &= ~km::Ctrl;
 	}
 	if  (
-		(( $code & 0xFF) >= ord(' ') || (( $code & 0xFF) == ord("\t"))) &&
+		( $code >= ord(' ') || ( $code == ord("\t"))) &&
 		(( $mod  & (km::Alt | km::Ctrl)) == 0) &&
 		(( $key == kb::NoKey) || ( $key == kb::Space) || ( $key == kb::Tab))
 	) {
