@@ -24,7 +24,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: apricot.h,v 1.184 2007/10/05 08:01:15 dk Exp $ */
+/* $Id: apricot.h,v 1.193 2008/11/06 10:54:37 dk Exp $ */
 
 #ifndef _APRICOT_H_
 #define _APRICOT_H_
@@ -241,14 +241,13 @@ extern "C" {
    #endif
 #endif
 
-#ifndef PTR2UV
-#define PTR2UV(x) ((UV)(x))
-#endif
-#ifndef PTR2IV
-#define PTR2IV(x) ((IV)(x))
-#endif
-#ifndef INT2PTR
-#define INT2PTR(type,x) ((type)((UV)x))
+#ifdef PTRV
+#undef PTR2UV
+#define PTR2UV(x) ((UV)(PTRV)(x))
+#undef PTR2IV
+#define PTR2IV(x) ((IV)(PTRV)(x))
+#undef INT2PTR
+#define INT2PTR(type,x) ((type)((PTRV)x))
 #endif
 
 #ifndef SvPV_nolen
@@ -342,7 +341,6 @@ typedef unsigned short Handle;
 #error "Cannot find adequate integer type"
 #endif
 typedef Handle ApiHandle;
-typedef long Color;
 
 #include "Types.h"
 
@@ -386,6 +384,7 @@ typedef unsigned long uint64_t;
 #endif
 #endif
 
+typedef uint32_t        Color;
 
 typedef uint8_t         Byte;
 typedef int16_t         Short;
@@ -424,6 +423,10 @@ typedef struct { double r,  ph; } TrigDComplex;
 
 /* Event structures */
 
+#ifdef KeyEvent
+#undef KeyEvent
+#endif
+
 typedef struct _KeyEvent {
    int    cmd;
    int    subcmd;
@@ -434,6 +437,10 @@ typedef struct _KeyEvent {
    int    repeat;
 } KeyEvent, *PKeyEvent;
 
+#ifdef PositionalEvent
+#undef PositionalEvent
+#endif
+
 typedef struct _PositionalEvent {
    int    cmd;
    int    subcmd;
@@ -444,6 +451,10 @@ typedef struct _PositionalEvent {
    Bool   dblclk;
 } PositionalEvent, *PPositionalEvent;
 
+#ifdef GenericEvent
+#undef GenericEvent
+#endif
+
 typedef struct _GenericEvent {
    int    cmd;
    int    subcmd;
@@ -1291,6 +1302,9 @@ push_hv_for_REDEFINED( SV **sp, HV *hv);
 extern int
 pop_hv_for_REDEFINED( SV **sp, int count, HV *hv, int shouldBe);
 
+extern void
+perl_error(void);
+
 extern void*
 create_object( const char *objClass, const char *types, ...);
 
@@ -1305,7 +1319,7 @@ SvBOOL( SV *sv)
 #endif
 
 #define pexist( key) hv_exists( profile, # key, strlen( #key))
-#define pdelete( key) hv_delete( profile, # key, strlen( #key), G_DISCARD)
+#define pdelete( key) (void) hv_delete( profile, # key, strlen( #key), G_DISCARD)
 #define dPROFILE  SV ** temporary_prf_Sv
 #define pget_sv( key) ((( temporary_prf_Sv = hv_fetch( profile, # key, strlen( # key), 0)) == nil) ? croak( "Panic: bad profile key (``%s'') requested in ``%s'', line %d\n", # key, __FILE__, __LINE__ ), &sv_undef : *temporary_prf_Sv)
 #define pget_i( key)  ( pget_sv( key), SvIV( *temporary_prf_Sv))
@@ -1314,7 +1328,7 @@ SvBOOL( SV *sv)
 #define pget_H( key)  gimme_the_mate( pget_sv( key))
 #define pget_B( key)  ( SvTRUE( pget_sv( key)))
 
-#define pset_sv_noinc( key, value) hv_store( profile, # key, strlen( # key), value, 0);
+#define pset_sv_noinc( key, value) (void)hv_store( profile, # key, strlen( # key), value, 0)
 #define pset_sv( key, value) pset_sv_noinc( key, newSVsv( value))
 #define pset_i( key, value)  pset_sv_noinc( key, newSViv( value))
 #define pset_f( key, value)  pset_sv_noinc( key, newSVnv( value))
@@ -1832,6 +1846,9 @@ extern long   apcError;
 *  apc functions   *
 ***************** */
 
+extern char *
+apc_last_error();
+
 extern Handle
 apc_get_application(void);
 
@@ -1887,6 +1904,9 @@ apc_application_go( Handle self);
 extern Bool
 apc_application_lock( Handle self);
 
+extern Bool
+apc_application_sync( void);
+
 extern Bool
 apc_application_unlock( Handle self);
 
@@ -2428,7 +2448,7 @@ apc_show_message( const char* message, Bool utf8);
 
 
 /* graphics constants */
-#define ARGB(r,g,b) ((long)(((unsigned char)(b)|((unsigned short)((unsigned char)(g))<<8))|(((unsigned long)(unsigned char)(r))<<16)))
+#define ARGB(r,g,b) ((uint32_t)(((unsigned char)(b)|((uint32_t)((unsigned char)(g))<<8))|(((uint32_t)((unsigned char)(r)))<<16)))
 
 /* colors */
 #define CL(const_name) CONSTANT(cl,const_name)
@@ -2467,37 +2487,37 @@ CL(Yellow)
 CL(White)
 #define    clGray             ARGB(128,128,128)
 CL(Gray)
-#define    clSysFlag          (unsigned long)(0x80000000)
+#define    clSysFlag          (uint32_t)(0x10000000)
 CL(SysFlag)
-#define    clSysMask          (unsigned long)(0x7FFFFFFF)
+#define    clSysMask          (uint32_t)(0xEFFFFFFF)
 CL(SysMask)
-#define    clInvalid          (unsigned long)(0x80000000)
+#define    clInvalid          (uint32_t)(0x10000000)
 CL(Invalid)
-#define    clNormalText       (unsigned long)(0x80000001)
+#define    clNormalText       (uint32_t)(0x10000001)
 CL(NormalText)
-#define    clFore             (unsigned long)(0x80000001)
+#define    clFore             (uint32_t)(0x10000001)
 CL(Fore)
-#define    clNormal           (unsigned long)(0x80000002)
+#define    clNormal           (uint32_t)(0x10000002)
 CL(Normal)
-#define    clBack             (unsigned long)(0x80000002)
+#define    clBack             (uint32_t)(0x10000002)
 CL(Back)
-#define    clHiliteText       (unsigned long)(0x80000003)
+#define    clHiliteText       (uint32_t)(0x10000003)
 CL(HiliteText)
-#define    clHilite           (unsigned long)(0x80000004)
+#define    clHilite           (uint32_t)(0x10000004)
 CL(Hilite)
-#define    clDisabledText     (unsigned long)(0x80000005)
+#define    clDisabledText     (uint32_t)(0x10000005)
 CL(DisabledText)
-#define    clDisabled         (unsigned long)(0x80000006)
+#define    clDisabled         (uint32_t)(0x10000006)
 CL(Disabled)
-#define    clLight3DColor     (unsigned long)(0x80000007)
+#define    clLight3DColor     (uint32_t)(0x10000007)
 CL(Light3DColor)
-#define    clDark3DColor      (unsigned long)(0x80000008)
+#define    clDark3DColor      (uint32_t)(0x10000008)
 CL(Dark3DColor)
-#define    clSet              (unsigned long)(0x80000009)
+#define    clSet              (uint32_t)(0x10000009)
 CL(Set)
-#define    clClear            (unsigned long)(0x8000000A)
+#define    clClear            (uint32_t)(0x1000000A)
 CL(Clear)
-#define    clMaxSysColor      (unsigned long)(0x8000000A)
+#define    clMaxSysColor      (uint32_t)(0x1000000A)
 CL(MaxSysColor)
 END_TABLE(cl,UV)
 #undef CL
@@ -2847,6 +2867,8 @@ AM(None)
 AM(MaskColor)
 #define    amAuto                2
 AM(Auto)
+#define    amMaskIndex           3
+AM(MaskIndex)
 END_TABLE(am,UV)
 #undef AM
 
